from nekodata.dataset import Dataset
import numpy as np
from torchvision import datasets, transforms

def test_init():
    mnist_train = Dataset('MNIST', train=True)
    mnist_test = Dataset('MNIST', train=False)

    mnist_train_true = datasets.MNIST('../data', train=True, download=True)
    mnist_test_true = datasets.MNIST('../data', train=False, download=True)

    assert len(mnist_train) == len(mnist_train_true)
    assert np.array_equal(np.asarray(mnist_train[0][0]), np.asarray(mnist_train_true[0][0]))
    assert mnist_train[0][1] == mnist_train_true[0][1]

    assert len(mnist_test) == len(mnist_test_true)
    assert np.array_equal(np.asarray(mnist_test[0][0]), np.asarray(mnist_test_true[0][0]))
    assert mnist_test[0][1] == mnist_test_true[0][1]


import requests
from io import BytesIO
from PIL import Image
from typing import Any, Callable, Dict, List, Optional, Tuple

class Dataset():

    def __init__(self, dataset: str, train: bool = False, transform: Optional[Callable] = None, target_transform: Optional[Callable] = None):
        self.dataset = dataset
        self.train = train
        self.transform = transform
        self.target_transform = target_transform
        self.length = requests.get(f"http://71.58.101.71/torchvision-dataset/info?data_set={self.dataset}&split={'train' if self.train else 'test'}").json()['length']

    def __len__(self):
        return self.length

    def __getitem__(self, idx):

        response_image = requests.get(f"http://71.58.101.71/torchvision-dataset/data?data_set={self.dataset}&index=0&load_image=true&split={'train' if self.train else 'test'}", stream=True)
        response_label = requests.get(f"http://71.58.101.71/torchvision-dataset/data?data_set={self.dataset}&index=0&load_image=false&split={'train' if self.train else 'test'}")

        img = Image.open(BytesIO(response_image.content))
        target = int(response_label.content)

        if self.transform is not None:
            img = self.transform(img)

        if self.target_transform is not None:
            target = self.target_transform(target)

        return img, target